# -*- coding: utf-8 -*-
from celery.task import PeriodicTask
from datetime import timedelta
from shiori.bookmark.agents import feed
from shiori.core.settings import CELERY_TIMEDELTA_MINUTES


class ProcessRunner(PeriodicTask):

    run_every = timedelta(minutes=CELERY_TIMEDELTA_MINUTES)

    def run(self, **kwargs):
        feed.register_bookmarks()



# -*- coding: utf-8 -*-
from django.db import models
from django.db.models.signals import pre_save
from django.contrib.auth.models import User
from shortuuidfield import ShortUUIDField
import jsonfield
from shiori.bookmark.agents.feed_parser import FeedParser
from shiori.bookmark import validators


class BaseObject(models.Model):
    id = ShortUUIDField(primary_key=True, auto=True, verbose_name='UUID')

    class Meta:
        abstract = True


class Category(BaseObject):
    category = models.CharField(max_length=255, unique=True)

    class Meta:
        db_table = 'category'

    def __unicode__(self):
        return self.category


class Tag(BaseObject):
    tag = models.CharField(max_length=255, unique=True)

    class Meta:
        db_table = 'tag'

    def __unicode__(self):
        return self.tag


class Bookmark(BaseObject):
    url = models.URLField()
    title = models.CharField(max_length=255)
    category = models.ForeignKey(Category)
    tags = models.ManyToManyField(Tag, through='BookmarkTag')
    registered_datetime = models.DateTimeField(auto_now=True,
                                               auto_now_add=True)
    description = models.TextField(blank=True)
    owner = models.ForeignKey(User)
    is_hide = models.BooleanField(default=False)

    class Meta:
        db_table = 'bookmark'
        unique_together = ('url', 'owner')

    def __unicode__(self):
        return self.title

    def get_absolute_url(self):
        return "/shiori/b/%s" % self.id


class BookmarkTag(models.Model):
    id = models.AutoField(primary_key=True)
    bookmark = models.ForeignKey(Bookmark,
                                 db_column='bookmark_id',
                                 to_field='id')
    tag = models.ForeignKey(Tag,
                            db_column='tag_id',
                            to_field='id')

    class Meta:
        db_table = 'bookmark_tag'
        unique_together = ('bookmark', 'tag')


class FeedSubscription(BaseObject):
    url = models.URLField()
    name = models.CharField(max_length=255, editable=False)
    owner = models.ForeignKey(User)
    default_category = models.ForeignKey(Category)

    class Meta:
        db_table = 'feed_subscription'
        unique_together = ('url', 'owner')

    def __unicode__(self):
        return self.url


def update_title(sender, instance, **kwargs):
    if validators.validate_url(instance.url):
        instance.name = FeedParser(instance.url).title
    else:
        raise ValueError("Cannot insert and updating in model saving.")

pre_save.connect(update_title, sender=FeedSubscription)


class CrawlingHistory(BaseObject):
    feed = models.ForeignKey(FeedSubscription)
    result = jsonfield.JSONField()
    update_datetime = models.DateTimeField(auto_now=True,
                                           auto_now_add=True)

    class Meta:
        db_table = 'crawling_history'

    def __unicode__(self):
        return self.id



# -*- coding: utf-8 -*-
from django.core.urlresolvers import resolve
from django.test import TestCase, TransactionTestCase
from django.contrib.auth.models import User
from django.views.generic import RedirectView
from django.db import IntegrityError
import shiori.bookmark.views
from shiori.bookmark.models import Category, Tag, Bookmark
from shiori.bookmark.feed_generator import LatestEntries
import shiori_tests.tests.vars as v


class BookmarkTransactionTest(TransactionTestCase):
    fixtures = ['shiori_tests/test_data/dummy_users.json',
                'shiori_tests/test_data/dummy_data.json']

    def setUp(self):
        self.user0 = User.objects.get(pk=1)
        self.user1 = User.objects.get(pk=2)
        categories = Category.objects.all()
        self.category0 = categories[0]
        self.category1 = categories[1]
        tags = Tag.objects.all()
        self.tag0 = tags[0]
        self.tag1 = tags[1]
        self.bookmark = Bookmark.objects.create(url=v.url0,
                                                title=v.title0,
                                                category=self.category0,
                                                description=v.description0,
                                                owner=self.user0)

    def test_get_categories(self):
        query = Category.objects.get(category=self.category0.category)
        self.assertEqual(query.__unicode__(), self.category0.category)
        self.assertEqual(query.__str__(), self.category0.category)

    def test_save_category_confilict(self):
        with self.assertRaises(IntegrityError):
            query = Category(category=self.category0.category)
            query.save()

    def test_save_category(self):
        query = Category(category=v.category2)
        query.save()

        query2 = Category.objects.get(category=v.category2)
        self.assertEqual(query.__str__(), v.category2)

    def test_update_category(self):
        query = Category(id=self.category0.id, category=v.category2)
        query.save()

        query2 = Category.objects.get(category=v.category2)
        self.assertEqual(query2.id, self.category0.id)

    def test_delete_category(self):
        self.category0.delete()
        with self.assertRaises(Category.DoesNotExist):
            Category.objects.get(category=self.category0.category)

        with self.assertRaises(Category.DoesNotExist):
            Category.objects.get(id=self.category0.id)

    def test_get_tag(self):
        query = Tag.objects.get(tag=self.tag0.tag)
        self.assertEqual(query.__unicode__(), self.tag0.tag)
        self.assertEqual(query.__str__(), self.tag0.tag)

    def test_save_tag_confilict(self):
        with self.assertRaises(IntegrityError):
            query = Tag(tag=self.tag1.tag)
            query.save()

    def test_save_tag(self):
        query = Tag(tag=v.tag2)
        query.save()

        query2 = Tag.objects.get(tag=v.tag2)
        self.assertEqual(query2.__str__(), v.tag2)

    def test_update_tag(self):
        query = Tag(id=self.tag0.id, tag=v.tag2)
        query.save()
        query2 = Tag.objects.get(tag=v.tag2)
        self.assertEqual(query2.id, self.tag0.id)

    def test_delete_tag(self):
        self.tag0.delete()
        with self.assertRaises(Tag.DoesNotExist):
            Tag.objects.get(tag=self.tag0.tag)

        with self.assertRaises(Tag.DoesNotExist):
            Tag.objects.get(id=self.tag0.id)

    def test_get_bookmark(self):
        query = Bookmark.objects.get(url=v.url0)
        self.assertEqual(query.__str__(), v.title0)

    def test_save_bookmark_conflict(self):
        with self.assertRaises(IntegrityError):
            query = Bookmark(url=v.url0,
                             title=v.title0,
                             category=self.category0,
                             description=v.description0,
                             owner=self.user0)
            query.save()

    def test_save_bookmark_same_url_by_another_user(self):
        query = Bookmark(url=v.url0,
                         title=v.title0,
                         category=self.category0,
                         description=v.description0,
                         owner=self.user1)
        query.save()
        query = Bookmark.objects.get(url=v.url0, owner=self.user1)
        self.assertEqual(query.__str__(), v.title0)

    def test_update_bookmark(self):
        query = Bookmark(id=self.bookmark.id,
                         url=v.url1,
                         title=v.title1,
                         category=self.category1,
                         description=v.description1,
                         owner=self.user0)
        query.save()

        query2 = Bookmark.objects.get(url=v.url1, owner=self.user0)
        self.assertEqual(query2.id, self.bookmark.id)

    def test_delete_bookmark(self):
        self.bookmark.delete()
        with self.assertRaises(Bookmark.DoesNotExist):
            Bookmark.objects.get(url=v.url0, owner=self.user0)

        with self.assertRaises(Bookmark.DoesNotExist):
            Bookmark.objects.get(id=self.bookmark.id, owner=self.user0)


class BookmarkTest(TestCase):

    def setUp(self):
        self.user = User.objects.create_user(v.username0,
                                             v.email0,
                                             v.password0)

    def test_shiori_url_resolve(self):
        found = resolve('/shiori/')
        self.assertEqual(found.func, shiori.bookmark.views.index)

    def test_profile_url_resolve(self):
        found = resolve('/shiori/profile/')
        self.assertEqual(found.func, shiori.bookmark.views.profile)

    def test_add_url_resolve(self):
        found = resolve('/shiori/add/')
        self.assertEqual(found.func, shiori.bookmark.views.add)

    def test_categories_url_resolve(self):
        found = resolve('/shiori/categories')
        self.assertEqual(found.func, shiori.bookmark.views.categories)

    def test_category_url_resolve(self):
        found = resolve('/shiori/categories/dummy_id')
        self.assertEqual(found.func, shiori.bookmark.views.category)

    def test_tags_url_resolve(self):
        found = resolve('/shiori/tags')
        self.assertEqual(found.func, shiori.bookmark.views.tags)

    def test_tag_url_resolve(self):
        found = resolve('/shiori/tags/dummy_id')
        self.assertEqual(found.func, shiori.bookmark.views.tag)

    def test_feeds_url_resolve(self):
        response = self.client.get('/shiori/feeds')
        self.assertEqual(response.status_code, 200)

    def test_bookmark_url_resolve(self):
        found = resolve('/shiori/b/dummy_id')
        self.assertEqual(found.func, shiori.bookmark.views.bookmark)

    def test_index_view(self):
        request = self.client.get('/shiori/')
        request.user = self.user
        response = shiori.bookmark.views.index(request)
        self.assertContains(response,
                            'logout',
                            status_code=200)

    def test_profile_view(self):
        request = self.client.get('/shiori/profile')
        request.user = self.user
        response = shiori.bookmark.views.profile(request)
        self.assertContains(response,
                            'date joined',
                            status_code=200)

    def test_categories_view(self):
        request = self.client.get('/shiori/categories')
        request.user = self.user
        response = shiori.bookmark.views.categories(request)
        self.assertContains(response,
                            'categories_list',
                            status_code=200)

    def test_category_view(self):
        request = self.client.get('/shiori/category/dummy_id')
        request.user = self.user
        response = shiori.bookmark.views.category(request, 'dummy_id')
        self.assertContains(response,
                            'category_view',
                            status_code=200)

    def test_tags_view(self):
        request = self.client.get('/shiori/tags')
        request.user = self.user
        response = shiori.bookmark.views.tags(request)
        self.assertContains(response,
                            'tags_list',
                            status_code=200)

    def test_tag_view(self):
        request = self.client.get('/shiori/tag/dummy_id')
        request.user = self.user
        response = shiori.bookmark.views.tag(request, 'dummy_id')
        self.assertContains(response,
                            'tag_view',
                            status_code=200)

    def test_bookmark_view(self):
        request = self.client.get('/shiori/b/dummy_id')
        request.user = self.user
        response = shiori.bookmark.views.bookmark(request, 'dummy_id')
        self.assertContains(response,
                            'bookmark_view',
                            status_code=200)



# -*- coding: utf-8 -*-
username0 = 'guest'
email0 = 'guest@example.org'
password0 = 'passw0rd'
username1 = 'guest1'
email1 = 'guest1@example.org'
password1 = 'passw1rd'
username2 = 'admin0'
email2 = 'admin0@example.org'
password2 = 'passw2rd'
url0 = 'http://dummy.example.org'
title0 = 'dummy タイトル'
description0 = 'This description is dummy sentense.'
url1 = 'http://dummy1.example.org'
title1 = 'dummy タイトル1'
description1 = 'This description is dummy1 sentense.'
url2 = 'http://dummy2.example.org'
title2 = 'dummy タイトル2'
description2 = 'This description is dummy2 sentense.'
category0 = 'test 0'
category1 = 'テスト 0'
category2 = 'test 1 abc'
tag0 = 'test 0'
tag1 = 'テスト 0'
tag2 = 'test 1 abc'



# -*- coding: utf-8 -*-
from django.core.urlresolvers import resolve
from django.test import TestCase
from django.contrib.auth.models import User
import unittest
import json

from shiori.api.views import CategoryViewSet
import shiori_tests.tests.vars as v


class APITest(TestCase):

    def setUp(self):
        User.objects.create_user(v.username0, v.email0, v.password0)
        User.objects.create_user(v.username1, v.email1, v.password1)
        User.objects.create_superuser(v.username2, v.email2, v.password2)
        self.client.login(username=v.username0, password=v.password0)

        payload = {'category': v.category0}
        self.client.post('/api/categories',
                         content_type='application/json',
                         data=json.dumps(payload))

        payload = {'tag': v.tag0}
        self.client.post('/api/tags',
                         content_type='application/json',
                         data=json.dumps(payload))

        payload = {'tag': v.tag1}
        self.client.post('/api/tags',
                         content_type='application/json',
                         data=json.dumps(payload))

        payload = {'url': v.url0,
                   'title': v.title0,
                   'category': v.category0,
                   'description': v.description0,
                   'is_hide': False}
        self.client.post('/api/bookmarks',
                         content_type='application/json',
                         data=json.dumps(payload))
        payload = {'url': v.url1,
                   'title': v.title1,
                   'category': v.category0,
                   'description': v.description1,
                   'is_hide': True}
        self.client.post('/api/bookmarks',
                         content_type='application/json',
                         data=json.dumps(payload))
        payload = {'bookmark': v.url0, 'tag': v.tag0}
        self.client.post('/api/bookmark_tags',
                         content_type='application/json',
                         data=json.dumps(payload))

    def test_api_categories_url_resolve(self):
        response = self.client.get('/api/categories')
        self.assertContains(response,
                            'results',
                            status_code=200)

    def test_api_tags_url_resolve(self):
        response = self.client.get('/api/tags')
        self.assertContains(response,
                            'results',
                            status_code=200)

    def test_api_bookmarks_url_resolve(self):
        response = self.client.get('/api/bookmarks')
        self.assertContains(response,
                            'results',
                            status_code=200)

    def test_api_bookmark_tags_url_resolve(self):
        response = self.client.get('/api/bookmark_tags')
        self.assertContains(response,
                            'results',
                            status_code=200)

    def test_post_tag(self):
        payload = {'tag': v.tag2}
        response = self.client.post('/api/tags',
                                    content_type='application/json',
                                    data=json.dumps(payload))
        self.assertEqual(response.status_code, 201)

    def test_post_tag_conflict(self):
        payload = {'tag': v.tag0}
        response = self.client.post('/api/tags',
                                    content_type='application/json',
                                    data=json.dumps(payload))
        self.assertContains(response,
                            '{"tag": ["Tag with this Tag already exists."]}',
                            status_code=400)

    def test_get_tags(self):
        response = self.client.get('/api/tags')
        self.assertContains(response,
                            '"tag": "%s"' % v.tag0,
                            status_code=200)

    def test_get_tag(self):
        r = self.client.get('/api/tags')
        id = json.loads(r.content).get('results')[0].get('id')
        response = self.client.get('/api/tags/%s' % id)
        self.assertContains(response,
                            '"tag": "%s"' % v.tag0,
                            status_code=200)

    def test_cannot_delete_tag(self):
        r = self.client.get('/api/tags')
        id = json.loads(r.content).get('results')[0].get('id')
        response = self.client.delete('/api/tags/%s' % id)
        self.assertEqual(response.status_code, 403)

    def test_cannot_put_tag(self):
        r = self.client.get('/api/tags')
        id = json.loads(r.content).get('results')[0].get('id')
        payload = {'id': id, 'tag': v.tag2}
        response = self.client.put('/api/tags/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload))
        self.assertContains(response,
                            '{"detail": "You do not have permission to'
                            ' perform this action."}',
                            status_code=403)

    def test_get_tags_by_anonymous(self):
        self.client.logout()
        r = self.client.get('/api/tags')
        self.assertContains(r,
                            'results',
                            status_code=200)

        id = json.loads(r.content).get('results')[0].get('id').encode('utf-8')
        response = self.client.get('/api/tags/%s' % id)
        self.assertContains(response,
                            '"tag": "%s"' % v.tag0,
                            status_code=200)

    def test_post_tags_by_anonymous(self):
        self.client.logout()
        payload = {'tag': v.tag1}
        response = self.client.post('/api/tags',
                                    content_type='application/json',
                                    data=json.dumps(payload))
        self.assertEqual(response.status_code, 403)

    def test_delete_tag_by_anonymous(self):
        self.client.logout()

        r = self.client.get('/api/tags')
        id = json.loads(r.content).get('results')[0].get('id')
        response = self.client.delete('/api/tags/%s' % id)
        self.assertEqual(response.status_code, 403)

    def test_put_tag_by_anonymous(self):
        self.client.logout()
        r = self.client.get('/api/tags')
        id = json.loads(r.content).get('results')[0].get('id')
        payload = {'id': id, 'tag': v.tag1}
        response = self.client.put('/api/tags/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload))
        self.assertEqual(response.status_code, 403)

    # [Fixed] ToDo: delete is allowed by admin user only.
    def test_delete_tag_by_superuser(self):
        self.client.logout()
        self.client.login(username=v.username2, password=v.password2)
        r = self.client.get('/api/tags')
        id = json.loads(r.content).get('results')[0].get('id')
        response = self.client.delete('/api/tags/%s' % id)
        self.assertEqual(response.status_code, 204)

    # [Fixed] ToDo: put is allowed by admin user or anyone does not use.
    def test_put_tag_by_superuser(self):
        self.client.logout()
        self.client.login(username=v.username2, password=v.password2)
        r = self.client.get('/api/tags')
        id = json.loads(r.content).get('results')[0].get('id')
        payload = {'id': id, 'tag': v.tag2}
        response = self.client.put('/api/tags/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload))
        self.assertEqual(response.status_code, 200)

    def test_post_category(self):
        payload = {'category': v.category1}
        response = self.client.post('/api/categories',
                                    content_type='application/json',
                                    data=json.dumps(payload))
        self.assertContains(response,
                            '"category":',
                            status_code=201)

    def test_get_categories(self):
        response = self.client.get('/api/categories')
        self.assertContains(response,
                            '"category": "%s"' % v.category0,
                            status_code=200)

    def test_get_category(self):
        r = self.client.get('/api/categories')
        id = json.loads(r.content).get('results')[0].get('id')
        response = self.client.get('/api/categories/%s' % id)
        self.assertContains(response,
                            '"category": "%s"' % v.category0,
                            status_code=200)

    def test_cannot_delete_category(self):
        r = self.client.get('/api/categories')
        id = json.loads(r.content).get('results')[0].get('id')
        response = self.client.delete('/api/categories/%s' % id)
        self.assertEqual(response.status_code, 403)

    def test_cannot_put_category(self):
        r = self.client.get('/api/categories')
        id = json.loads(r.content).get('results')[0].get('id')
        payload = {'id': id, 'category': v.category1}
        response = self.client.put('/api/categories/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload))
        self.assertContains(response,
                            '{"detail": "You do not have permission to'
                            ' perform this action."}',
                            status_code=403)

    def test_get_categories_by_anonymous(self):
        self.client.logout()
        response = self.client.get('/api/categories')
        self.assertContains(response,
                            'results',
                            status_code=200)

    def test_get_category_by_anonymous(self):
        self.client.logout()
        r = self.client.get('/api/categories')
        id = json.loads(r.content).get('results')[0].get('id').encode('utf-8')
        response = self.client.get('/api/categories/%s' % id)
        self.assertContains(response,
                            '"category": "%s"' % v.category0,
                            status_code=200)

    def test_post_category_by_anonymous(self):
        self.client.logout()
        payload = {'category': v.category1}
        response = self.client.post('/api/categories',
                                    content_type='application/json',
                                    data=json.dumps(payload))
        self.assertEqual(response.status_code, 403)

    def test_delete_category_by_anonymous(self):
        self.client.logout()
        r = self.client.get('/api/categories')
        id = json.loads(r.content).get('results')[0].get('id')
        response = self.client.delete('/api/categories/%s' % id)
        self.assertEqual(response.status_code, 403)

    def test_put_category_by_anonymous(self):
        self.client.logout()
        r = self.client.get('/api/categories')
        id = json.loads(r.content).get('results')[0].get('id')
        payload = {'id': id, 'category': v.category1}
        response = self.client.put('/api/categories/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload))
        self.assertEqual(response.status_code, 403)

    # [Fixed] ToDo: delete is allowed by admin user or anyone does not use.
    def test_delete_category_by_superuser(self):
        self.client.logout()
        self.client.login(username=v.username2, password=v.password2)
        r = self.client.get('/api/categories')
        id = json.loads(r.content).get('results')[0].get('id')
        response = self.client.delete('/api/categories/%s' % id)
        self.assertEqual(response.status_code, 204)

    # [Fixed] ToDo: put is allowed by admin user or anyone does not use.
    def test_put_category_by_superuser(self):
        self.client.logout()
        self.client.login(username=v.username2, password=v.password2)
        r = self.client.get('/api/categories')
        id = json.loads(r.content).get('results')[0].get('id')
        payload = {'id': id, 'category': v.category1}
        response = self.client.put('/api/categories/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload))
        self.assertEqual(response.status_code, 200)

    def test_post_bookmark_without_category(self):
        payload = {'url': v.url2,
                   'title': v.title2,
                   'category': '',
                   'description': v.description2,
                   'is_hide': False}
        response = self.client.post('/api/bookmarks',
                                    content_type='application/json',
                                    data=json.dumps(payload))
        self.assertEqual(response.status_code, 400)

    def test_post_bookmark_without_is_hide(self):
        payload = {'url': v.url2,
                   'title': v.title2,
                   'category': v.category0,
                   'description': v.description2,
                   'is_hide': None}
        response = self.client.post('/api/bookmarks',
                                    content_type='application/json',
                                    data=json.dumps(payload))
        self.assertContains(response,
                            '"url": "%s"' % v.url2,
                            status_code=201)

    def test_post_bookmark_without_url(self):
        payload = {'url': '',
                   'title': v.title2,
                   'category': v.category0,
                   'description': v.description2,
                   'is_hide': False}
        response = self.client.post('/api/bookmarks',
                                    content_type='application/json',
                                    data=json.dumps(payload))
        self.assertEqual(response.status_code, 400)

    def test_post_bookmark_without_title(self):
        payload = {'url': v.url2,
                   'title': '',
                   'category': v.category0,
                   'description': v.description2,
                   'is_hide': False}
        response = self.client.post('/api/bookmarks',
                                    content_type='application/json',
                                    data=json.dumps(payload))
        self.assertEqual(response.status_code, 400)

    def test_post_bookmark_without_description(self):
        payload = {'url': v.url2,
                   'title': v.url2,
                   'category': v.category0,
                   'description': '',
                   'is_hide': False}
        response = self.client.post('/api/bookmarks',
                                    content_type='application/json',
                                    data=json.dumps(payload))
        self.assertContains(response,
                            '"url": "%s"' % v.url2,
                            status_code=201)

    def test_post_bookmark(self):
        payload = {'url': v.url2,
                   'title': v.title2,
                   'category': v.category0,
                   'description': v.description2,
                   'is_hide': False}
        response = self.client.post('/api/bookmarks',
                                    content_type='application/json',
                                    data=json.dumps(payload))
        self.assertContains(response,
                            '"url": "%s"' % v.url2,
                            status_code=201)

    def test_delete_bookmark(self):
        r = self.client.get('/api/bookmarks')
        id = json.loads(r.content).get('results')[0].get('id')
        response = self.client.delete('/api/bookmarks/%s' % id)
        self.assertEqual(response.status_code, 204)

    def test_put_bookmark(self):
        r = self.client.get('/api/bookmarks')
        id = json.loads(r.content).get('results')[0].get('id')
        payload = {'id': id,
                   'url': v.url2,
                   'title': v.title2,
                   'category': v.category0,
                   'description': v.description2,
                   'is_hide': True}
        response = self.client.put('/api/bookmarks/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload))
        self.assertContains(response,
                            '"url": "%s"' % v.url2,
                            status_code=200)

    # ToDo: bookmark_tags is required owner
    def test_post_bookmark_tags(self):
        payload = {'bookmark': v.url0,
                   'tag': v.tag1}
        response = self.client.post('/api/bookmark_tags',
                                    content_type='application/json',
                                    data=json.dumps(payload))
        self.assertContains(response,
                            '"bookmark": "%s"' % v.url0,
                            status_code=201)

    def test_delete_bookmark_tags(self):
        r = self.client.get('/api/bookmark_tags')
        id = json.loads(r.content).get('results')[0].get('id')
        response = self.client.delete('/api/bookmark_tags/%s' % id)
        self.assertEqual(response.status_code, 204)

    def test_put_bookmark_tags(self):
        r = self.client.get('/api/bookmark_tags')
        id = json.loads(r.content).get('results')[0].get('id')
        payload = {'id': id,
                   'bookmark': v.url0,
                   'tag': v.tag1}
        response = self.client.put('/api/bookmark_tags/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload))
        self.assertContains(response,
                            '"bookmark": "%s"' % v.url0,
                            status_code=200)

    def test_post_bookmark_by_anonymous(self):
        self.client.logout()
        payload = {'url': v.url2,
                   'title': v.title2,
                   'category': v.category0,
                   'description': v.description2,
                   'is_hide': False}
        response = self.client.post('/api/bookmarks',
                                    content_type='application/json',
                                    data=json.dumps(payload))
        self.assertEqual(response.status_code, 403)

    def test_delete_bookmark_by_anonymous(self):
        self.client.logout()
        r = self.client.get('/api/bookmarks')
        id = json.loads(r.content).get('results')[0].get('id')
        response = self.client.delete('/api/bookmarks/%s' % id)
        self.assertEqual(response.status_code, 403)

    def test_put_bookmark_by_anonymous(self):
        self.client.logout()
        r = self.client.get('/api/bookmarks')
        id = json.loads(r.content).get('results')[0].get('id')
        payload = {'id': id,
                   'url': v.url0,
                   'title': v.title1,
                   'category': v.category0,
                   'description': v.description1,
                   'is_hide': True}

        response = self.client.put('/api/bookmarks/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload))
        self.assertEqual(response.status_code, 403)

    # [Fixed] ToDo: delete is allowed by owner user only.
    def test_delete_bookmark_by_another_user(self):
        self.client.logout()
        self.client.login(username=v.username1, password=v.password1)
        r = self.client.get('/api/bookmarks?is_all=true')
        id = json.loads(r.content).get('results')[0].get('id')
        response = self.client.delete('/api/bookmarks/%s' % id)
        self.assertEqual(response.status_code, 404)

        response = self.client.delete('/api/bookmarks/%s?is_all=true' % id)
        self.assertEqual(response.status_code, 403)

    # ToDo: put is allowed by owner user only.
    # Url of bookmark is not unique in all users,
    # but url by users should be unique.
    @unittest.skip("ToDo skipping")
    def test_put_bookmark_by_another_user(self):
        self.client.logout()
        self.client.login(username=v.username1, password=v.password1)

        r = self.client.get('/api/bookmarks')
        id = json.loads(r.content).get('results')[0].get('id')

        payload = {'id': id,
                   'url': v.url0,
                   'title': v.title1,
                   'category': v.category0,
                   'description': v.description1,
                   'is_hide': True}
        response = self.client.put('/api/bookmarks/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload))
        self.assertEqual(response.status_code, 403)

    def test_post_bookmark_tags_by_anonymous(self):
        self.client.logout()
        payload = {'bookmark': v.url0,
                   'tag': v.tag1}
        response = self.client.post('/api/bookmark_tags',
                                    content_type='application/json',
                                    data=json.dumps(payload))
        self.assertEqual(response.status_code, 403)

    def test_delete_bookmark_tags_by_anonymous(self):
        self.client.logout()

        r = self.client.get('/api/bookmark_tags')
        id = json.loads(r.content).get('results')[0].get('id')

        response = self.client.delete('/api/bookmark_tags/%s' % id)
        self.assertEqual(response.status_code, 403)

    def test_put_bookmark_tags_by_anonymous(self):
        self.client.logout()
        r = self.client.get('/api/bookmark_tags')
        id = json.loads(r.content).get('results')[0].get('id')
        payload = {'id': id,
                   'bookmark': v.url0,
                   'tag': v.tag1}

        response = self.client.put('/api/bookmark_tags/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload))
        self.assertEqual(response.status_code, 403)

    # ToDo: delete is allowed by owner user only.
    @unittest.skip("ToDo skipping")
    def test_delete_bookmark_tags_by_another_user(self):
        self.client.logout()
        self.client.login(username=v.username1, password=v.password1)

        r = self.client.get('/api/bookmark_tags')
        id = json.loads(r.content).get('results')[0].get('id')
        response = self.client.delete('/api/bookmark_tags/%s' % id)
        self.assertEqual(response.status_code, 403)

    # ToDo: put is allowed by owner user only.
    @unittest.skip("ToDo skipping")
    def test_put_bookmark_tags_by_another_user(self):
        self.client.logout()
        self.client.login(username=v.username1, password=v.password1)

        r = self.client.get('/api/bookmark_tags')
        id = json.loads(r.content).get('results')[0].get('id')
        payload = {'id': id,
                   'bookmark': v.url0,
                   'tag': v.tag1}
        response = self.client.put('/api/bookmark_tags/%s' % id,
                                   content_type='application/json',
                                   data=json.dumps(payload))
        self.assertEqual(response.status_code, 403)



# -*- coding: utf-8 -*-
import os
import json
import shortuuid
from datetime import datetime


def generate_data(key):
    return {
        "model": "bookmark.%s" % key,
        "pk": shortuuid.uuid(),
        "fields": {
            "%s" % key: shortuuid.uuid()
        }
    }


def generate_bookmark(owner_id, category_id, is_hide):
    return {
        "model": "bookmark.bookmark",
        "pk": shortuuid.uuid(),
        "fields": {
            "url": "http://%s.example.org/%s" % (shortuuid.uuid()[:4],
                                                 shortuuid.uuid()),
            "title": shortuuid.uuid(),
            "category": category_id,
            "description": shortuuid.uuid(),
            "owner": owner_id,
            "is_hide": is_hide
        }
    }


def bookmark_tag(id, bookmark_id, tag_id):
    return {
        "model": "bookmark.bookmark_tag",
        "pk": id,
        "fields": {
            "bookmark": bookmark_id,
            "tag": tag_id
        }
    }


def feed_subscription(owner_id, category_id):
    return {
        "model": "bookmark.feed_subscription",
        "pk": shortuuid.uuid(),
        "fields": {
            "url": "http://%s.example.org/%s" % (shortuuid.uuid()[:4],
                                                 shortuuid.uuid()),
            "name": shortuuid.uuid(),
            "owner": owner_id,
            "default_category": category_id
        }
    }


def crawling_history(feed_id):
    return {
        "model": "bookmark.crawling_history",
        "pk": shortuuid.uuid(),
        "fields": {
            "id": shortuuid.uuid(),
            "feed": feed_id,
            "update_datetime": datetime.now().strftime(
                "%Y-%m-%dT%H:%M:%S+00:00")
        }
    }


def generate_json():
    data_list = []
    for i in range(10):
        data_list.append(generate_data("category"))
        data_list.append(generate_data("tag"))
    return json.dumps(data_list)


def main():
    dirpath = os.path.dirname(os.path.abspath(__file__))
    json_path = os.path.join(dirpath, 'dummy_data.json')

    with open(json_path, 'w') as f:
        f.write(generate_json())


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from django.conf.urls import patterns, include, url
from django.views.generic import RedirectView
from django.contrib import admin
admin.autodiscover()


urlpatterns = patterns('',
                       (r'', include('django_openid_auth.urls')),
                       (r'^$', RedirectView.as_view(url='/shiori/')),
                       (r'^logout/$', 'django.contrib.auth.views.logout',
                        {'template_name': 'bookmark/logout.html'}),
                       (r'^api/', include('shiori.api.urls')),
                       (r'^shiori/', include('shiori.bookmark.urls')),
                       url(r'^admin/', include(admin.site.urls)),)



# -*- coding: utf-8 -*-
from django.db.models import Q
from django.utils.html import escape
from django.contrib.auth.models import AnonymousUser
from rest_framework import viewsets
from rest_framework.response import Response
from rest_framework.permissions import (IsAuthenticated,
                                        IsAuthenticatedOrReadOnly,
                                        IsAdminUser)
from rest_framework.exceptions import NotAuthenticated
from shiori.bookmark.models import (Category,
                                    Tag,
                                    Bookmark,
                                    BookmarkTag,
                                    FeedSubscription)
from shiori.api.permissions import (IsOwnerOrReadOnly,
                                    IsAuthenticatedAndCreateReadOnly)
from shiori.api.serializers import (CategorySerializer,
                                    TagSerializer,
                                    BookmarkSerializer,
                                    BookmarkTagSerializer,
                                    FeedSubscriptionSerializer)


class CategoryViewSet(viewsets.ModelViewSet):
    queryset = Category.objects.all()
    serializer_class = CategorySerializer
    permission_classes = (IsAuthenticatedAndCreateReadOnly,)

    def pre_save(self, obj):
        obj.category = escape(self.request.DATA.get('category'))

    def get_queryset(self):
        user = self.request.user
        if user.is_superuser:
            return self.queryset
        elif isinstance(user, AnonymousUser):
            categories = [category.get('category')
                          for category
                          in Bookmark.objects.values('category').distinct()]
            return self.queryset.filter(id__in=categories)
        else:
            if self.request.QUERY_PARAMS.get('is_all') == 'true':

                categories = [category.get('category')
                              for category
                              in Bookmark.objects.values('category')
                              .filter(Q(owner=user) | Q(is_hide=False))
                              .distinct()]
            else:
                categories = [category.get('category')
                              for category
                              in Bookmark.objects.values('category')
                              .filter(owner=user).distinct()]
            return self.queryset.filter(id__in=categories)


class TagViewSet(viewsets.ModelViewSet):
    queryset = Tag.objects.all()
    serializer_class = TagSerializer
    permission_classes = (IsAuthenticatedAndCreateReadOnly,)
    paginate_by = 50

    def pre_save(self, obj):
        obj.tag = escape(self.request.DATA.get('tag'))

    def get_queryset(self):
        user = self.request.user
        if user.is_superuser:
            return self.queryset
        elif isinstance(user, AnonymousUser):
            bookmark = [bookmark.get('id')
                        for bookmark
                        in Bookmark.objects.values('id')
                        .filter(is_hide=False)]
            return (self.queryset.filter(bookmarktag__bookmark__in=bookmark)
                    .distinct())
        else:
            # authenticated user
            if self.request.QUERY_PARAMS.get('is_all') == 'true':
                bookmark = [bookmark.get('id')
                            for bookmark
                            in Bookmark.objects.values('id')
                            .filter(Q(owner=user) | Q(is_hide=False))]
            else:
                bookmark = [bookmark.get('id')
                            for bookmark
                            in Bookmark.objects.values('id')
                            .filter(owner=user)]
            return self.queryset\
                       .filter(bookmarktag__bookmark__in=bookmark).distinct()


class BookmarkViewSet(viewsets.ModelViewSet):
    queryset = Bookmark.objects.all()
    serializer_class = BookmarkSerializer
    permission_classes = (IsOwnerOrReadOnly,)

    def pre_save(self, obj):
        if isinstance(self.request.user, AnonymousUser):
            raise NotAuthenticated
        else:
            obj.owner = self.request.user
        obj.title = escape(self.request.DATA.get('title'))
        obj.description = escape(self.request.DATA.get('description'))

    def get_queryset(self):
        user = self.request.user
        if isinstance(user, AnonymousUser):
            # anonymous can read all bookmarks except is_hide=True in default.
            _q = self.set_filter(Q(is_hide=False))
        elif user.is_superuser:
            # superuser can read all bookmarks in default.
            _q = self.set_filter(Q())
        else:
            # general users can read own bookmarks only in default.
            # they can read other users bookmarks when is_all=true.
            if self.request.QUERY_PARAMS.get('is_all') == 'true':
                _q = self.set_filter(Q(is_hide=False) | Q(owner=user))
            else:
                _q = self.set_filter(Q(owner=user))
        if self.request.QUERY_PARAMS.get('tag'):
            try:
                tag = Tag.objects.get(tag=self.request.QUERY_PARAMS.get('tag'))
                self.queryset = tag.bookmark_set.all()
            except Tag.DoesNotExist as e:
                print(e)
        return self.queryset.filter(_q)

    def set_filter(self, q_obj):
        category = None
        if self.request.QUERY_PARAMS.get('category'):
            try:
                category = Category.objects.get(
                    category=self.request.QUERY_PARAMS.get('category'))
            except Category.DoesNotExist as e:
                category = None

        if category:
            q_obj = q_obj & Q(category=category)

        if self.request.QUERY_PARAMS.get('search'):
            search = self.request.QUERY_PARAMS.get('search')
            q_obj = (q_obj & Q(title__icontains=search)
                     | Q(description__icontains=search))

        return q_obj


class BookmarkTagViewSet(viewsets.ModelViewSet):
    queryset = BookmarkTag.objects.all()
    serializer_class = BookmarkTagSerializer
    permission_classes = (IsAuthenticatedOrReadOnly,)


class FeedSubscriptionViewSet(viewsets.ModelViewSet):
    queryset = FeedSubscription.objects.all()
    serializer_class = FeedSubscriptionSerializer
    permission_classes = (IsAuthenticated,)

    def pre_save(self, obj):
        if isinstance(self.request.user, AnonymousUser):
            raise NotAuthenticated
        else:
            obj.owner = self.request.user

    def get_queryset(self):
        return FeedSubscription.objects.filter(owner=self.request.user)



# -*- coding: utf-8 -*-
from django.core.urlresolvers import resolve
from django.test import TestCase
from django.views.generic import RedirectView
import django_openid_auth
import django.contrib.auth.views


class CoreTest(TestCase):

    def test_root_url_resolve(self):
        response = self.client.get('/')
        self.assertRedirects(response, '/shiori/',
                             status_code=301, target_status_code=200)

    def test_login_url_resolv(self):
        found = resolve('/login/')
        self.assertEqual(found.func, django_openid_auth.views.login_begin)

    def test_logout_url_resolv(self):
        found = resolve('/logout/')
        self.assertEqual(found.func, django.contrib.auth.views.logout)



# -*- coding: utf-8 -*-
"""
see follow url
http://www.django-rest-framework.org/api-guide/permissions#custom-permissions
"""
from rest_framework import permissions


class IsOwnerOrReadOnly(permissions.BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method in permissions.SAFE_METHODS:
            return True

        return obj.owner == request.user


class IsAuthenticatedAndCreateReadOnly(permissions.BasePermission):
    def has_permission(self, request, view):
        if request.method in permissions.SAFE_METHODS:
            return True
        elif request.method == 'POST':
            if request.user.is_authenticated():
                return True
        else:
            if request.user.is_superuser:
                return True



# -*- coding: utf-8 -*-
from django.contrib.auth.models import User
from django.db import IntegrityError
import json
from shiori.bookmark.models import (Bookmark,
                                    Category,
                                    FeedSubscription,
                                    CrawlingHistory)
from shiori.bookmark.agents.feed_parser import FeedParser


def register_bookmarks():

    for feed in FeedSubscription.objects.all():
        result = fetch_feeds(url=feed.url,
                             category=feed.default_category,
                             owner=feed.owner)
        CrawlingHistory(feed=feed, result=json.dumps(result)).save()


def fetch_feeds(**kwargs):
    result = []
    for entry in FeedParser(kwargs.get('url')).retrieve_items():
        rc, msg = add_item(url=entry.get('link'),
                           title=entry.get('title'),
                           category=kwargs.get('category'),
                           owner=kwargs.get('owner'))
        if rc is False and "already registered:" in msg:
            break
        result.append(dict(link=entry.get('link'),
                           rc=rc,
                           msg=msg))
    return result


def add_categories(url):
    [add_category(entry.get('category'))
     for entry
     in FeedParser(url).retrieve_items()
     if entry.get('category')]


def add_item(**kwargs):
    category = Category.objects.get(category=kwargs.get('category'))
    owner = User.objects.get(username=kwargs.get('owner'))

    bookmark = Bookmark(url=kwargs.get('url'),
                        title=kwargs.get('title'),
                        category=category,
                        owner=kwargs.get('owner'))
    try:
        bookmark.save()
        return True, ''
    except IntegrityError as error:
        return False, "already registered: %s" % error



# -*- coding: utf-8 -*-
from __future__ import absolute_import

import os
from celery import Celery
from django.conf import settings

from shiori.core.settings import LOGIN_URL

os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'shiori.core.settings')

app = Celery('core')
app.config_from_object('django.conf:settings')
app.autodiscover_tasks(lambda: settings.INSTALLED_APPS)


@app.task(bind=True)
def debug_task(self):
    print('Request: {0!r}'.format(self.request))



# -*- coding: utf-8 -*-
from defusedxml import lxml
from lxml.etree import XMLSyntaxError
import requests
import sys


def get_updated(etree, nsmap):
    if etree.find('.//%slastBuildDate' % get_ns(nsmap)) is not None:
        return etree.find('.//%slastBuildDate' % get_ns(nsmap)).text
    elif etree.find('.//%supdated' % get_ns(nsmap)) is not None:
        return etree.find('.//%supdated' % get_ns(nsmap)).text
    elif etree.find('.//%spublished' % get_ns(nsmap)) is not None:
        return etree.find('.//%spublished' % get_ns(nsmap)).text
    elif etree.find('.//%spubDate' % get_ns(nsmap)) is not None:
        return etree.find('.//%spubDate' % get_ns(nsmap)).text
    elif etree.find('.//%sdate' % get_ns(nsmap, 'dc')) is not None:
        return etree.find('.//%sdate' % get_ns(nsmap, 'dc')).text


def get_title(etree, ns):
    if etree.find('.//%stitle' % ns) is not None:
        return etree.find('.//%stitle' % ns).text


def get_link(etree, ns):
    if etree.find('.//%slink' % ns) is not None:
        if etree.find('.//%slink' % ns).get('href'):
            return etree.find('.//%slink' % ns).get('href')
        elif etree.find('.//%slink' % ns).text is not None:
            return etree.find('.//%slink' % ns).text


def get_items(etree, ns):
    if etree.find('.//%schannel/%sitem' % (ns, ns)) is not None:
        return etree.findall('.//%schannel/%sitem' % (ns, ns))
    elif etree.find('.//%sentry' % ns) is not None:
        return etree.findall('.//%sentry' % ns)
    elif etree.find('.//%sitem' % ns) is not None:
        return etree.findall('.//%sitem' % ns)


def get_ns(nsmap, key=None):
    if nsmap.get(key) is None:
        ns = ''
    else:
        ns = '{%s}' % nsmap.get(key)
    return ns


class FeedParser(object):

    def __init__(self, url):
        try:
            response = requests.get(url, stream=True)
        except requests.ConnectionError as e:
            print(e)
            sys.exit(1)
        try:
            etree = lxml.fromstring(response.content)
        except XMLSyntaxError as e:
            print(e)
            sys.exit(1)
        self.nsmap = etree.nsmap
        self.ns = get_ns(etree.nsmap, None)

        self.updated = get_updated(etree, self.nsmap)
        self.title = get_title(etree, self.ns)
        self.items = get_items(etree, self.ns)
        self.etree = etree

    def retrieve_items(self):
        return [dict(title=get_title(item, self.ns),
                     link=get_link(item, self.ns),
                     updated=get_updated(item, self.nsmap))
                for item in self.items]



# -*- coding: utf-8 -*-
import csv
import json
import os
import sys

sys.path.insert(0, os.path.dirname(os.path.dirname(
    os.path.dirname(os.path.abspath(__file__)))))
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "shiori_tests.settings")
from django.contrib.auth.models import User


def convert_format(row):
    return {
        "model": "auth.user",
        "pk": int(row[0]),
        "fields": {
            "first_name": row[1],
            "last_name": row[2],
            "username": row[4],
            "email": row[3],
            "is_active": True,
            "is_staff": False,
            "is_superuser": False,
            "password": convert_hash_password(row[5])
        }
    }


def convert_hash_password(plaintext_password):
    u = User()
    u.set_password(plaintext_password)
    return u.password


def main():
    dirpath = os.path.dirname(os.path.abspath(__file__))
    csv_path = os.path.join(dirpath, 'FakeNameGenerator.com_2a6e893c.csv')
    json_path = os.path.join(dirpath, 'dummy_users.json')
    if sys.version_info < (3, 0):
        with open(csv_path, 'rb') as f:
            reader = csv.reader(f)
            with open(json_path, 'w') as f:
                f.write(json.dumps([convert_format(row) for row in reader
                                    if row[0] != '\xef\xbb\xbfNumber']))
    else:
        with open(csv_path, 'rt', encoding='utf-8') as f:
            reader = csv.reader(f)
            with open(json_path, 'w') as f:
                f.write(json.dumps([convert_format(row) for row in reader
                                    if row[0] != '\ufeffNumber']))


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from django.contrib import admin
from shiori.bookmark.models import (Bookmark,
                                    Category,
                                    Tag,
                                    BookmarkTag,
                                    FeedSubscription,
                                    CrawlingHistory)


class BookmarkAdmin(admin.ModelAdmin):
    list_display = ('title', 'category', 'registered_datetime',
                    'owner', 'is_hide')


class BookmarkTagAdmin(admin.ModelAdmin):
    list_display = ('bookmark', 'tag')


class FeedSubscriptionAdmin(admin.ModelAdmin):
    list_display = ('name', 'owner', 'default_category')


class CrawlingHistoryAdmin(admin.ModelAdmin):
    list_display = ('get_name', 'update_datetime', 'result')

    def get_name(self, obj):
        return obj.feed.name


admin.site.register(Bookmark, BookmarkAdmin)
admin.site.register(Category)
admin.site.register(Tag)
admin.site.register(BookmarkTag, BookmarkTagAdmin)
admin.site.register(FeedSubscription, FeedSubscriptionAdmin)
admin.site.register(CrawlingHistory, CrawlingHistoryAdmin)



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Removing unique constraint on 'Bookmark', fields ['url']
        db.delete_unique('bookmark', ['url'])

        # Adding unique constraint on 'Bookmark', fields ['url', 'owner']
        db.create_unique('bookmark', ['url', 'owner_id'])


    def backwards(self, orm):
        # Removing unique constraint on 'Bookmark', fields ['url', 'owner']
        db.delete_unique('bookmark', ['url', 'owner_id'])

        # Adding unique constraint on 'Bookmark', fields ['url']
        db.create_unique('bookmark', ['url'])


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'bookmark.bookmark': {
            'Meta': {'unique_together': "(('url', 'owner'),)", 'object_name': 'Bookmark', 'db_table': "'bookmark'"},
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['bookmark.Category']"}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('shortuuidfield.fields.ShortUUIDField', [], {'max_length': '22', 'primary_key': 'True'}),
            'is_hide': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'owner': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']"}),
            'registered_datetime': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'auto_now_add': 'True', 'blank': 'True'}),
            'tags': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['bookmark.Tag']", 'through': u"orm['bookmark.BookmarkTag']", 'symmetrical': 'False'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200'})
        },
        u'bookmark.bookmarktag': {
            'Meta': {'unique_together': "(('bookmark', 'tag'),)", 'object_name': 'BookmarkTag', 'db_table': "'bookmark_tag'"},
            'bookmark': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['bookmark.Bookmark']", 'db_column': "'bookmark_id'"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'tag': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['bookmark.Tag']", 'db_column': "'tag_id'"})
        },
        u'bookmark.category': {
            'Meta': {'object_name': 'Category', 'db_table': "'category'"},
            'category': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'id': ('shortuuidfield.fields.ShortUUIDField', [], {'max_length': '22', 'primary_key': 'True'})
        },
        u'bookmark.tag': {
            'Meta': {'object_name': 'Tag', 'db_table': "'tag'"},
            'id': ('shortuuidfield.fields.ShortUUIDField', [], {'max_length': '22', 'primary_key': 'True'}),
            'tag': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['bookmark']


# -*- coding: utf-8 -*-
from django.core.exceptions import ValidationError
import sys
if sys.version_info > (3, 0):
    from urllib.parse import urlparse
else:
    from urlparse import urlparse
from netaddr import IPAddress, AddrFormatError
from socket import gaierror, gethostbyname, getaddrinfo, AF_INET6
from shiori.core.settings import FEED_EXCLUDE_FQDN


def validate_url(value):
    hostname = urlparse(value).netloc
    if ':' in hostname:
        hostname = hostname.split(':')[0]
    if hostname in FEED_EXCLUDE_FQDN:
        raise ValidationError('%s is prohibited in FEED_EXCLUDE_FQDN.' % value)
    try:
        if IPAddress(hostname).is_loopback():
            raise ValidationError('loopback address is prohibited.')
    except AddrFormatError:
        if getaddr(hostname) is None:
            raise ValidationError('%s is not found.' % value)
        for ip in getaddr(hostname):
            if IPAddress(ip).is_loopback():
                raise ValidationError('loopback address is prohibited.')
    return True


def getaddr(hostname):
    addresses = []
    try:
        addresses.append(gethostbyname(hostname))
    except TypeError as e:
        print(e)
    except gaierror as e:
        print(e)
    try:
        addresses.append(getaddrinfo(hostname, None, AF_INET6)[0][4][0])
    except gaierror as e:
        print(e)
    return addresses



# -*- coding: utf-8 -*-
from django.conf.urls import patterns, include, url
from rest_framework import routers
from shiori.api import views

router = routers.DefaultRouter(trailing_slash=False)
router.register(r'categories', views.CategoryViewSet)
router.register(r'tags', views.TagViewSet)
router.register(r'bookmarks', views.BookmarkViewSet)
router.register(r'bookmark_tags', views.BookmarkTagViewSet)
router.register(r'feed_subscription', views.FeedSubscriptionViewSet)

urlpatterns = patterns('shiori.api.views',
                       url(r'^', include(router.urls)),
                       )



# -*- coding: utf-8 -*-
from django.contrib.syndication.views import Feed
from django.core.urlresolvers import reverse
from shiori.bookmark.models import Bookmark


class LatestEntries(Feed):
    title = 'Shiori new bookmarks'
    link = '/shiori/'
    description = 'Updates on changes and additions to Shiori.'
    description_template = 'feeds/latest_title.html'

    def items(self):
        return Bookmark.objects.order_by('-registered_datetime')[:5]
